#!/bin/perl
# usage = perl join_overlapping_entries_VSGs_gff3.pl VSG_gff3 > VSG_overlapping_joined_output.gff3
# Input file is the gff3 output generated with Blast_output_to_gff3_v2.pl from the BLAST output file from the Cross et al. (2014) VSG list to our genome. --> Sorted by chromosome and then by start position!
# Output is a gff3 file in which overlapping VSG entries were merged into one.Joining the notes and names.

use strict;
use warnings;

my $VSG_gff = $ARGV[0];

$" = "\t"; # Setting the output field separator to the tab character.	
my $last;

## Merge overlapping VSG features.. are there some? Yes, there are quite a bit. Like ~350.

open(my $VSG_in, '<', $VSG_gff) or die "Could not open file '$VSG_gff' $!";

while(my $VSG_row = <$VSG_in>){

chomp $VSG_row;
my ($VSG_Chrom, $VSG_Source, $VSG_Feature, $VSG_Start, $VSG_End, $VSG_Score, $VSG_Strand, $VSG_Frame, $VSG_Attributes) = split("\t",$VSG_row);
my $VSG_Name;
my $VSG_Note;
	if($VSG_Chrom =~ m/^#/ ) {
	next;
	}

	if($last) {# Test if last overlaps with the new entry and merge them in last.
		if ($last->[0] ne $VSG_Chrom || $last->[4] < $VSG_Start) {# If the chromosome of the current entry is not equal to the previous one OR the end of the previous entry is smaller than the start of the current one, print the previous entry.
		print "@$last\n";
		}
		else {# If the ranges between the previous entry and the current one overlap, assign the start of the previous to $VSG_Start, If the current entry END is smaller than the previous, assign the previous one to $VSG_End,and add the annotation of the current one to $VSG_Attributes 
			
			#Break the Attributes of the last entry ($last->[8]) into $VSG_Name and $VSG_Note
			
			$last->[8] =~ m/(Name=(.*);)/;
			my $Last_Name = $1;
			$Last_Name =~ s/;$//;
			$last->[8] =~ m/(Note=(.*))/;
			my $Last_Note = $1;
			
			$VSG_Attributes =~ m/(Name=(.*);)/; # Match the "Name= until the ;" and store it in $1 variable
			$VSG_Name = $1 =~ s/Name=//r; # Assign to $VSG_Name the $1 variable but removing "Name="
			
			$VSG_Attributes =~ m/(Note=(.*))/; # Match the "Note= until the end of the line" and store it in $1 variable
			$VSG_Note = $1 =~ s/Note=//r; # Assign to $VSG_Note the $1 variable but removing "Note="
#			$VSG_Note = "From $VSG_Start to $VSG_End, $VSG_Note"; # Modifiy $VSG_Note adding the coordinates"

			$VSG_Attributes = "$Last_Name, $VSG_Name"." $Last_Note"." $VSG_Note";

			# Assign the start of the previous to $VSG_Start and if the current entry END is smaller than the previous, assign the previous one to $VSG_End

			$VSG_Start = $last->[3];
			$VSG_End = $last->[4] if $VSG_End < $last->[4];
		}

	}
	$last = [$VSG_Chrom, $VSG_Source, $VSG_Feature, $VSG_Start, $VSG_End, $VSG_Score, $VSG_Strand, $VSG_Frame, $VSG_Attributes];
}
print "@$last\n";
close($VSG_in) or die "Could not close filehandle '$VSG_in' $!";
exit;
